/* Copyright (c) 2013-2022 Jeffrey Pfau
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
#ifndef M_SCRIPT_LUA_H
#define M_SCRIPT_LUA_H
#include <mgba/script/context.h>

extern struct mScriptEngine2* const mSCRIPT_ENGINE_LUA;

#endif
